@@ -347,7 +347,7 @@ void IndividualAggregator::addContactDone(const GError *gerror,
     try {
         // Handle result of folks_persona_store_add_persona_from_details().
         if (!persona || gerror) {
-            GErrorCXX::throwError("add contact", gerror);
+            GErrorCXX::throwError(SE_HERE, "add contact", gerror);
         }
 
         const gchar *uid = folks_persona_get_uid(persona);
@@ -422,7 +422,7 @@ void IndividualAggregator::removeContactDone(const GError *gerror,
 {
     try {
         if (gerror) {
-            GErrorCXX::throwError("remove contact", gerror);
+            GErrorCXX::throwError(SE_HERE, "remove contact", gerror);
         }
         result.done();
     } catch (...) {
@@ -488,7 +488,7 @@ void IndividualAggregator::runWithAddressBookPrepared(const GError *gerror,
     try {
         // Called after EDS system store is prepared, successfully or unsuccessfully.
         if (gerror) {
-            GErrorCXX::throwError("prepare EDS system address book", gerror);
+            GErrorCXX::throwError(SE_HERE, "prepare EDS system address book", gerror);
         }
         operation();
     } catch (...) {
@@ -539,7 +539,6 @@ void IndividualAggregator::doRunWithPersona(const boost::function<void (FolksPer
 class FolksTest : public CppUnit::TestFixture {
     CPPUNIT_TEST_SUITE(FolksTest);
     CPPUNIT_TEST(open);
-    CPPUNIT_TEST(asyncError);
     CPPUNIT_TEST(gvalue);
     CPPUNIT_TEST_SUITE_END();
 
@@ -671,29 +670,6 @@ private:
         CPPUNIT_ASSERT(str2.get() == fixed);
     }
 
-    void asyncError() {
-        bool done = false, failed = false;
-        SYNCEVO_GLIB_CALL_ASYNC(folks_individual_aggregator_remove_individual,
-                                boost::bind(asyncCB, _1,
-                                            "folks_individual_aggregator_remove_individual",
-                                            boost::ref(failed), boost::ref(done)),
-                                NULL, NULL);
-        while (!done) {
-            g_main_context_iteration(NULL, true);
-        }
-        // Invalid parameters are not reported!
-        CPPUNIT_ASSERT(!failed);
-
-        // using simpler macro
-        GErrorCXX gerror;
-        SYNCEVO_GLIB_CALL_SYNC(NULL,
-                               gerror,
-                               folks_individual_aggregator_remove_individual,
-                               NULL, NULL);
-        // Invalid parameters are not reported!
-        CPPUNIT_ASSERT(!gerror);
-    }
-
     static void individualSignal(std::ostringstream &out,
                                  const char *action,
                                  int index,
