@@ -56,6 +56,7 @@ MapSyncSource::MapSyncSource(const SyncSourceParams &params,
         m_operations.m_backupData = m_sub->getOperations().m_backupData;
         m_operations.m_restoreData = m_sub->getOperations().m_restoreData;
     }
+    m_operations.m_isEmpty = m_sub->getOperations().m_isEmpty;
 }
 
 void MapSyncSource::enableServerMode()
@@ -178,7 +179,7 @@ void MapSyncSource::detectChanges(SyncSourceRevisions::ChangeMode mode)
         break;
     default:
         // ?!
-        throwError("unknown change mode");
+        throwError(SE_HERE, "unknown change mode");
         break;
     }
 
